#version 150

uniform sampler2D texture;
// tell GPU, that texture map is actually a shadow map
uniform sampler2DShadow ShadowMap;

uniform sampler2D normalmap;

// calculate view vec of light?
uniform mat4 LightView;
uniform mat4 view;
uniform vec3 lightPosition;
uniform vec4 lightColor;
uniform vec4 ambientColor;
uniform vec4 fogColor;
uniform vec4 specularColor;
uniform float fogDensity;
uniform vec4 mistColor;
uniform float mistDensity;
uniform float mistLevel;
uniform float activeShadow;
uniform float useAlpha;
uniform float useNormalMap;
uniform float	ambientMat ;
uniform float	diffuseMat ;
uniform float	specularMat ;
uniform float	specularExp ;


in vec3 worldNormal;
in vec4 worldPosition;
in vec2 fragmentTexCoord;
in vec4 viewPosition;
in vec3 camVec;
in vec3 tangentSpace;
in vec3 bitangentSpace;
in vec3 bitangent;



// shadow stuff
in vec4 ShadowCoord; // passed on from VS
// end shadow stuff
	
out vec4 fragColor;



void main(void)
{
// shadow stuff


	/*
	 * grab texture  color. if alpha is < 1.0 discard
	 */
	vec4 texColor = texture2D(texture, fragmentTexCoord);

	
		if (useAlpha != 0.0f && texColor.a < 1.0 ){
		 discard;
		}
	
		

	vec4 tempColor = vec4(0.0f, 0.0f, 0.0f, 1.0f);
	
	//Renormalize and homogenize input variables
	vec3 normal = worldNormal;
	
	vec3 position = worldPosition.xyz / worldPosition.w;
	vec3 viewPos = viewPosition.xyz / viewPosition.w;
	
	//viewing direction
	vec3 viewDir = normalize(camVec - (-worldNormal));
	
	//light direction
	vec3 lightDir = normalize(lightPosition - position);
	vec3 tbnNormalDir;
	vec3 tbnLightDir;
	vec3 tbnViewDir;
	
	
	float NdotL; 
	float NdotH;
	float spec;
	float selfShadow = 1.0;
	
	vec3 halfwayDir;
	

	
	vec3 bumpcolor;
	// if we are using a normal map
	if(useNormalMap != 0.0){
		//use same coords as tex
	    bumpcolor = vec3(texture2D(normalmap,fragmentTexCoord.xy));
		//bring into -1 | 1 range
		bumpcolor = (bumpcolor -0.5) * 2.0;
		mat3 rotmat = mat3(tangentSpace,bitangentSpace,worldNormal);
		// move normal into tangent space
		normal = rotmat * bumpcolor;
		// parallax stuff http://graphics.cs.brown.edu/games/SteepParallax/index.html
		
		vec3 _tbnViewDir = rotmat * viewDir;
		tbnViewDir = _tbnViewDir;
		tbnLightDir = rotmat * lightDir;

		
		 float height = 1.0;
		 float numSteps = 20;
		 float bumpScale = 0.04;

		  vec2 offsetCoord = fragmentTexCoord.xy;

		vec4 NB = texture2D(normalmap, offsetCoord);
		vec3 tsE = normalize(tbnViewDir);
		numSteps = mix(numSteps*2, numSteps, tsE.z);
		float step;
		vec2 delta;
		step = 1.0 / numSteps;
		delta = vec2(-_tbnViewDir.x, _tbnViewDir.y) * bumpScale / (_tbnViewDir.z * numSteps);

		while (NB.a < height) {
        height -= step;
        offsetCoord += delta;
        NB = texture2D(normalmap, offsetCoord);
    }
		height = NB.a;
		texColor = texture2D(texture, offsetCoord);
		tbnLightDir = normalize(tbnLightDir);
		vec3 tbnNormalDir = NB.xyz * 2 - 1;
		vec3 tsH = normalize(tbnNormalDir + normalize(tbnViewDir));
		
		NdotL = max(0, dot(tbnNormalDir, tbnLightDir ));
		NdotH = max(0, dot(tbnNormalDir, tsH));
		spec = NdotH * NdotH;

		selfShadow = 0.0;
		
		if (NdotL > 0) {

        // Trace a shadow ray along the light vector.
		float numShadowSteps = mix(60,5,tbnLightDir.z);
		//float numShadowSteps = mix(80,10,tbnLightDir.z);
		
        step = 1.0 / numShadowSteps;
        delta = vec2(tbnLightDir.x, -tbnLightDir.y) * bumpScale / (numShadowSteps * tbnLightDir.z);

            // We start one iteration out to avoid shadow acne 
            // (could start bumped a little without going
            // a whole iteration).
		
            height = NB.a + step * 0.1;

            while ((NB.a < height) && (height < 1.0)) 
			{
                height += step;
                offsetCoord += delta;
                NB = texture2D(normalmap, offsetCoord);
            }
	
            // We are in shadow if we left the loop because
            // we hit a point
            if(NB.a < height){
				selfShadow = 1.0;
			}
			else {
			// dont use full self shadow
				selfShadow = 0.7;
			}

            // Shadows will make the whole scene darker, so up the light contribution
           // lightColor = lightColor * 1.2;
			
        }

	}
	else {
		
		tbnLightDir = lightDir;
			//halfway vector
		halfwayDir = normalize(lightDir + viewDir);
		NdotH = dot	(normal, halfwayDir);
		NdotL = dot(normal, lightDir);
		spec = max(0.0f, NdotH);
	}
	

    vec4 ambientLight = ambientColor * texColor;
	
    tempColor += ambientMat * ambientLight;

    	// don't add shadow to ambient term!
    	// perspective division, depth-comparison and PCF
    	// is implicitly carried out by the GPU
    	
    if (activeShadow != 0.0){
	
    	vec3 shadowLightVec = vec3(LightView[0][2],LightView[1][2],LightView[2][2]);
    	vec3 ShadowLightDir = normalize(shadowLightVec);
    	float shadow = textureProj(ShadowMap,ShadowCoord);
		// shadow is a discrete value so we set it to 
		// something arbitrary like 0.7 to milden the effect
		if (shadow <= 0.0){
			shadow = 0.1;
		}
    	if (dot(normal,ShadowLightDir) > 0.0f){
    		vec4 diffuseLight = lightColor * max(0.0f, NdotL) * texColor;
    		tempColor += diffuseMat * diffuseLight * shadow * selfShadow;
    		
        	vec4 specularLight = specularColor * pow(spec, specularExp) * 	texColor;
        	tempColor += specularMat * specularLight * shadow * selfShadow;
    		
    	}
    	else {
    		//tempColor = vec4(1.0,0.0,0.0,1.0);
    	}
    }
    else{
		vec4 diffuseLight = lightColor * max(0.0f, NdotL) * texColor;
		tempColor += diffuseMat * diffuseLight * selfShadow;
		
    	vec4 specularLight = specularColor * pow(spec, specularExp) * 					texColor;
    	tempColor += specularMat * specularLight * selfShadow ;
    }
    	
	
	
	//fog effect
	float fExp = exp(-pow(fogDensity, 2) * pow(viewPos.z, 				2));
	//ground mist effect
	float mExp = exp(-pow(mistDensity, 3) * pow(-position.y + 			mistLevel + 3, 3));

	if(position.y < mistLevel) {
		fragColor = mExp * (fExp * tempColor + (1 - 					fExp) * fogColor) + (1 - 	mExp) * 					mistColor;
	}
	else {
		fragColor = fExp * tempColor + (1 - fExp) * 					fogColor;
	}
	//fragColor = vec4(((normal / 2.0 ) + 0.5),1.0);
	//fragColor = vec4(bumpcolor,1.0);

}




