#version 150

uniform sampler2D texture;
// tell GPU, that texture map is actually a shadow map
uniform sampler2DShadow ShadowMap;

uniform sampler2D normalmap;

// calculate view vec of light?
uniform mat4 LightView;
uniform mat4 view;
uniform vec3 lightPosition;
uniform vec4 lightColor;
uniform vec4 ambientColor;
uniform vec4 fogColor;
uniform vec4 specularColor;
uniform float fogDensity;
uniform vec4 mistColor;
uniform float mistDensity;
uniform float mistLevel;
uniform float activeShadow;
uniform float useAlpha;
uniform float useNormalMap;
uniform float	ambientMat ;
uniform float	diffuseMat ;
uniform float	specularMat ;
uniform float	specularExp ;


in vec3 worldNormal;
in vec4 worldPosition;
in vec2 fragmentTexCoord;
in vec4 viewPosition;
in vec3 camVec;
in vec3 tangentSpace;
in vec3 bitangentSpace;
in vec3 bitangent;


// shadow stuff
in vec4 ShadowCoord; // passed on from VS
// end shadow stuff
	
out vec4 fragColor;



void main(void)
{
// shadow stuff


	/*
	 * grab texture  color. if alpha is < 1.0 discard
	 */
	vec4 texColor = texture2D(texture, fragmentTexCoord);

	
		if (useAlpha != 0.0f && texColor.a < 1.0 ){
		 discard;
		}
	
		

	vec4 tempColor = vec4(0.0f, 0.0f, 0.0f, 1.0f);
	
	//Renormalize and homogenize input variables
	vec3 normal = worldNormal;
	
	vec3 position = worldPosition.xyz / worldPosition.w;
	vec3 viewPos = viewPosition.xyz / viewPosition.w;
	
	
	//light direction
	vec3 lightDir = normalize(lightPosition - position);
	
	vec3 bumpcolor;
	// if we are using a normal map
	if(useNormalMap != 0.0){
		//use same coords as tex
	    bumpcolor = vec3(texture2D(normalmap,fragmentTexCoord.xy));
		//bring into -1 | 1 range
		bumpcolor = (bumpcolor -0.5) * 2.0;
		mat3 rotmat = mat3(tangentSpace,bitangentSpace,worldNormal);
		normal = rotmat * bumpcolor;
	}
	
	

	//viewing direction
	vec3 viewDir = normalize(camVec - (-worldNormal));


	//halfway vector
	vec3 halfwayDir = normalize(lightDir + viewDir);


    vec4 ambientLight = ambientColor * texColor;
    tempColor += ambientMat * ambientLight;

    	// don't add shadow to ambient term!
    	// perspective division, depth-comparison and PCF
    	// is implicitly carried out by the GPU
    	
    if (activeShadow != 0.0){
    	vec3 shadowLightVec = vec3(LightView[0][2],LightView[1][2],LightView[2][2]);
    	vec3 ShadowLightDir = normalize(shadowLightVec);
    	float shadow = textureProj(ShadowMap,ShadowCoord);
		
    	if (dot(normal,ShadowLightDir) > 0.0f){
    		vec4 diffuseLight = lightColor * max(0.0f, dot(normal, lightDir)) * texColor;
    		tempColor += diffuseMat * diffuseLight * shadow;
    		
        	vec4 specularLight = specularColor * pow(max(0.0f, dot	(normal, halfwayDir)), specularExp) * 					texColor;
        	tempColor += specularMat * specularLight * shadow;
    		
    	}
    	else {
    		//tempColor = vec4(1.0,0.0,0.0,1.0);
    	}
    }
    else{
		vec4 diffuseLight = lightColor * max(0.0f, dot					(normal, lightDir)) * texColor;
		tempColor += diffuseMat * diffuseLight ;
		
    	vec4 specularLight = specularColor * pow(max(0.0f, dot				(normal, halfwayDir)), specularExp) * 					texColor;
    	tempColor += specularMat * specularLight ;
    }
    	
	
	
	//fog effect
	float fExp = exp(-pow(fogDensity, 2) * pow(viewPos.z, 				2));
	//ground mist effect
	float mExp = exp(-pow(mistDensity, 3) * pow(-position.y + 			mistLevel + 3, 3));

	if(position.y < mistLevel) {
		fragColor = mExp * (fExp * tempColor + (1 - 					fExp) * fogColor) + (1 - 	mExp) * 					mistColor;
	}
	else {
		fragColor = fExp * tempColor + (1 - fExp) * 					fogColor;
	}
	//fragColor = vec4(((normal / 2.0 ) + 0.5),1.0);
	//fragColor = vec4(bumpcolor,1.0);
}




