#version 150

uniform sampler2D texture;
// tell GPU, that texture map is actually a shadow map
uniform sampler2DShadow ShadowMap;

uniform mat4 LightView;
uniform mat4 view;
uniform vec3 lightPosition;
uniform vec4 lightColor;
uniform vec4 ambientColor;
uniform vec4 fogColor;
uniform vec4 specularColor;
uniform float fogDensity;
uniform vec4 mistColor;
uniform float mistDensity;
uniform float mistLevel;
uniform float activeShadow;
uniform float useAlpha;
uniform float	ambientMat ;
uniform float	diffuseMat ;
uniform float	specularMat ;
uniform float	specularExp ;

in vec3 worldNormal;
in vec4 worldPosition;
in vec2 fragmentTexCoord;
in vec4 viewPosition;



// shadow stuff
	in vec4 ShadowCoord; // passed on from VS
// end shadow stuff
	
out vec4 fragColor;



void main(void)
{
// shadow stuff


	/*
	 * grab texture  color. if alpha is < 1.0 discard
	 */
		vec4 texColor = texture2D(texture, fragmentTexCoord);
	
		if ( useAlpha != 0.0f && texColor.a < 1.0){
			discard;
		}
	


	vec4 tempColor = vec4(0.0f, 0.0f, 0.0f, 1.0f);
	
	//Renormalize and homogenize input variables
	vec3 normal = normalize(worldNormal);
	vec3 position = worldPosition.xyz / worldPosition.w;
	vec3 viewPos = viewPosition.xyz / viewPosition.w;
	
	vec3 camVec = vec3(view[0][2],view[1][2],view[2][2]);
	vec3 shadowLightVec = vec3(LightView[0][2],LightView[1][2],LightView[2][2]);
			
	//light direction
	vec3 lightDir = normalize(lightPosition - position);
	//viewing direction
	vec3 viewDir = normalize(camVec - (-normal));
	vec3 ShadowLightDir = normalize(shadowLightVec);

	//halfway vector
	vec3 halfwayDir = normalize(lightDir + viewDir);


    	vec4 ambientLight = ambientColor * texColor;
    	tempColor += ambientMat * ambientLight;

    	// don't add shadow to ambient term!
    	// perspective division, depth-comparison and PCF
    	// is implicitly carried out by the GPU
    	 if (activeShadow != 0.0){
    	    	vec3 shadowLightVec = vec3(LightView[0][2],LightView[1][2],LightView[2][2]);
    	    	vec3 ShadowLightDir = normalize(shadowLightVec);
    	    	float shadow = textureProj(ShadowMap,ShadowCoord);
    			
    	    	if (dot(normal,ShadowLightDir) >= 0.0f){
    	    		vec4 diffuseLight = lightColor * max(0.0f, dot					(normal, lightDir)) * texColor;

			tempColor += diffuseMat * diffuseLight * shadow;
    	    		
    	        	vec4 specularLight = specularColor * pow(max(0.0f, dot				(normal, halfwayDir)), specularExp) * 					texColor;
    	        	tempColor += specularMat * specularLight * shadow;
    	    		
    	    	}
    	    	else {
    	    		//tempColor = vec4(1.0,0.0,0.0,1.0);
    	    	}
    	    }
    	    else{
    			vec4 diffuseLight = lightColor * max(0.0f, dot					(normal, lightDir)) * texColor;

			tempColor += diffuseMat * diffuseLight ;
    			
    	    	vec4 specularLight = specularColor * pow(max(0.0f, dot				(normal, halfwayDir)), specularExp) * 					texColor;
    	    	tempColor += specularMat * specularLight ;
    	    }
        	
	
	//fog effect
	float fExp = exp(-pow(fogDensity, 2) * pow(viewPos.z, 				2));
	//ground mist effect
	float mExp = exp(-pow(mistDensity, 3) * pow(-position.y + 			mistLevel + 3, 3));

	if(position.y < mistLevel) {
		fragColor = mExp * (fExp * tempColor + (1 - 					fExp) * fogColor) + (1 - 	mExp) * 					mistColor;
	}
	else {
		fragColor = fExp * tempColor + (1 - fExp) * 					fogColor;
	}
	
	

}




